//
// C++ Interface: Diagram
//
// Description: Diagramm Basisklasse
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _DIAGRAM_
#define _DIAGRAM_

#include <QWidget>
#include <QList>
#include <QStringList>
#include <QColor>
#include <QMouseEvent>
#include <QDataStream>

class Diagram : public QWidget
{
	Q_OBJECT
	public:
	Diagram(QWidget *parent=0);
	
	QVector<QAction*> actionList();
	
	virtual void saveData(QDataStream& s,int ver=1);
	virtual void loadData(QDataStream& s,int ver=1);
	
	public slots:
		virtual void setData(const QVector<double> &d);
		virtual void setLabels(const QStringList &l);
		virtual void setColLabels(const QStringList &l);
		virtual void setRowWidth(int w);
		virtual void setTitle(const QString &t);
		virtual void setValuePostfix(const QString& p);
	
	protected:
		void paintEvent( QPaintEvent *ev)=0;
		void mousePressEvent(QMouseEvent *event);
		void mouseMoveEvent(QMouseEvent *event);
		
	protected:
		QVector<double> data;
		QList<QColor> colors;
		QStringList labels,colLabels;
		QString value_postfix,_title;
		int row_width;
		QPoint dragStartPosition;
		QVector<QAction*> actions;
	
	protected slots:
		virtual void colorAction();
		virtual void fontAction();
};

#endif
