//
// C++ Implementation: haeufigkeitsmultidiagramadapter
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "haeufigkeitsmultidiagramadapter.h"
#include "diagram.h"
#include "columnselectdialog.h"
#include <QAction>
#include <QInputDialog>
#include <QStandardItemModel>
#include <QtDebug>
#include <map>

HaeufigkeitsMultiDiagramAdapter::HaeufigkeitsMultiDiagramAdapter()
 : ModelMultiDiagramAdapter(),filter_wert(),filter_spalte(-1),input_filter("")
{
	QAction *a = new QAction(tr("mit Spalte verknpfen"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(depAction()));
	actions << a;
	a = new QAction(tr("Verknpfung aufheben"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(undepAction()));
	actions << a;
}


HaeufigkeitsMultiDiagramAdapter::~HaeufigkeitsMultiDiagramAdapter()
{
}


void HaeufigkeitsMultiDiagramAdapter::updateDiagram()
{
	if(!td || !_model || Spalten.size()==0) return;
	using namespace std;
	
	QStringList cLabels;
	foreach(int i,Spalten)	cLabels << _model->headerData(i,Qt::Horizontal).toString();
	
	const int w = _model->rowCount();
	map<QString,int> he;
	
	for(QVector<int>::iterator it=Spalten.begin();it!=Spalten.end();++it)
	{
		for(int i=0;i<w;++i)
		{
			QString v = _model->data(_model->index(i,*it)).toString();
			he[v]++; // Haeufigkeit zaehlen
		}
	}
	
	QStringList labels;
	for(map<QString,int>::iterator it=he.begin();it!=he.end();++it)
	{
		labels << (*it).first;
	}
	
	const int lw = labels.count();
	QVector<double> sdata(Spalten.size() * lw);
	
	int j=0;
	QVector<double>::iterator a = sdata.begin();
	
	for(QVector<int>::iterator it=Spalten.begin();it!=Spalten.end();++it,++j)
	{
		map<QString,int> lhe;
		// Spaltenhaeufigkeit
		for(int i=0;i<w;++i)
		{
			QString v = _model->data(_model->index(i,*it)).toString();
			if( filter_spalte>=0 && filter_wert!=_model->data(_model->index(i,filter_spalte)).toString()) continue;
			if( !input_filter.isNull() && input_filter == _model->data(_model->index(i,*it))) continue;
			lhe[v]++; // Haeufigkeit zaehlen
		}
		
		for(int i=0;i<labels.count();++i)
		{
			sdata[i+j*lw]=lhe[labels[i]];
		}
		processData(a,a+lw);
		a+=lw;
	}
	
	td->setRowWidth(lw);
	td->setLabels(labels);
	td->setColLabels(cLabels);
	td->setData(sdata);
}

void HaeufigkeitsMultiDiagramAdapter::processData(QVector<double>::iterator b,QVector<double>::iterator e)
{
	
}

void HaeufigkeitsMultiDiagramAdapter::setDepFilter(QString Filter_Wert,int Filter_Spalte)
{
	filter_wert = Filter_Wert;
	filter_spalte = Filter_Spalte;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsMultiDiagramAdapter::setOutputFilter(const QString& fv)
 */
void HaeufigkeitsMultiDiagramAdapter::setOutputFilter(const QVariant& fv)
{
	output_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsMultiDiagramAdapter::setInputFilter(const QString& fv)
 */
void HaeufigkeitsMultiDiagramAdapter::setInputFilter(const QVariant& fv)
{
	input_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsMultiDiagramAdapter::depAction()
 */
void HaeufigkeitsMultiDiagramAdapter::depAction()
{
	bool ok;
	int col=-1;
	if(model()) col = ColumnSelectDialog::getColumn(model());
	if(col<0) return;
	QSet<QString> values;
	for(int i=0;i<model()->rowCount();++i) values << model()->data(model()->index(i,col)).toString();
	
	QString v = QInputDialog::getItem(this,tr("Filter"),tr("Wert nach dem gefiltert werden soll"),values.toList(),0,true,&ok);
	
	if(!ok) return;
	setDepFilter(v,col);
	updateDiagram();
}


/*!
    \fn HaeufigkeitsMultiDiagramAdapter::undepAction()
 */
void HaeufigkeitsMultiDiagramAdapter::undepAction()
{
	setDepFilter(QString(),-1);
}

void HaeufigkeitsMultiDiagramAdapter::saveData(QDataStream& s,int ver)
{
	if(ver>1) ModelMultiDiagramAdapter::saveData(s,ver);
	s << input_filter;
	s << filter_wert << (qint32) filter_spalte;
}

void HaeufigkeitsMultiDiagramAdapter::loadData(QDataStream& s,int ver)
{
	if(ver>1) ModelMultiDiagramAdapter::loadData(s,ver);
	s >> input_filter;
	QString v;
	int sp;
	s >> v >> sp;
	setDepFilter(v,sp);
}
