//
// C++ Implementation: modelaenlichkeitstabelle
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "modelaenlichkeitstabelle.h"
#include <QStandardItemModel>
#include <QtDebug>
#include <QLayout>
#include <QDrag>
#include <QMimeData>
#include <QApplication>

ModelAenlichkeitsTabelle::ModelAenlichkeitsTabelle()
 : ModelClient()
{
	table = new QTableWidget(this);
	QGridLayout *l = new QGridLayout(this);
	l->addWidget( table,0,0);	
}


ModelAenlichkeitsTabelle::~ModelAenlichkeitsTabelle()
{
}


void ModelAenlichkeitsTabelle::setSpalten( const QVector<int> &s)
{
	Spalten=s;
	updateDiagram();
}

void ModelAenlichkeitsTabelle::setModel(QStandardItemModel *m)
{
	if(_model) disconnect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex & )),this,SLOT(updateDiagram()));
	_model = m;
	connect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	updateDiagram();
}

void ModelAenlichkeitsTabelle::updateDiagram()
{
	if(!_model || Spalten.size()==0) return;
	
	int rows = _model->rowCount();
	table->setRowCount(rows);
	table->setColumnCount(rows);
	for(int i=0;i<rows;++i) table->setColumnWidth(i,50);

	// Werte berechnen
	using namespace std;
	
	map<QString, int>he;
	for(int i=0;i<_model->rowCount();++i)
	{
		for(int j=0; j<_model->rowCount();++j)
		{
			int v=0;
			for(QVector<int>::iterator it=Spalten.begin();it!=Spalten.end();++it)
			{
				QVariant p1 = _model->data(_model->index(i,*it));
				QVariant p2 = _model->data(_model->index(j,*it));
				
				if(p1 == p2) v++;
			}
			//in tabelle an i,j stellen
			int val = v*100/Spalten.size();
			QTableWidgetItem *newItem = new QTableWidgetItem(tr("%1").arg(val));
			newItem->setTextAlignment(Qt::AlignRight);
			if(val<10) newItem->setBackground(Qt::gray);
			else if(val<30) newItem->setBackground(Qt::lightGray);
			else if(val<50) newItem->setBackground(Qt::yellow);
			else if(val<70) newItem->setBackground(Qt::blue);
			else newItem->setBackground(Qt::green);
			table->setItem(i, j, newItem);
		}
	}
}

QString ModelAenlichkeitsTabelle::toText()
{
	QString out;
	for(int i=0;i<table->rowCount();++i)
	{
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			if(it)
			{
				if(j>0) out += "\t";
				out += it->text();
			}
		}
		out += "\n";
	}
	return out;
}

QString ModelAenlichkeitsTabelle::toHTML()
{
	QString out="<TABLE border=1 width=90%>";
	for(int i=0;i<table->rowCount();++i)
	{
		out += "<TR>";
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			out += "<TD bgcolor=\""+ it->backgroundColor().name() + "\">"+it->text()+"</TD>";
		}
		out += "</TR>";
	}
	out += "</TABLE>";
	return out;
}

// Drag and Drop
 void ModelAenlichkeitsTabelle::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
         dragStartPosition = event->pos();
 }


 void ModelAenlichkeitsTabelle::mouseMoveEvent(QMouseEvent *event)
 {
     if (!(event->buttons() & Qt::LeftButton))
         return;
     if ((event->pos() - dragStartPosition).manhattanLength()
          < QApplication::startDragDistance())
         return;

     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;

	TextInhalt *ti = dynamic_cast<TextInhalt*>(this);
	if(ti)
	{
		// Get Text
		mimeData->setHtml(ti->toHTML());
	}
	else
	{
     // Get Image
     QPixmap px = QPixmap::grabWidget(this);
     mimeData->setImageData(px.toImage());
	}
     //mimeData->setData(mimeType, data);
     drag->setMimeData(mimeData);

     Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
 }
