//
// C++ Implementation: StatistikTabelle
//
// Description:
//
//
// Author: Priyanka Singh <priyanka@india-souring-research.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "statistiktabelle.h"
#include <numeric>
#include <QStandardItemModel>
#include <QtDebug>
#include <QLayout>
#include <QTableWidget>
#include <QDrag>
#include <QMimeData>
#include <QApplication>
#include <QAction>
#include <QMenu>
#include <QInputDialog>

#include "modelclient.h"
#include "columnselectdialog.h"

StatistikTabelle::StatistikTabelle()
	:ModelClient(),input_filter(""),filter_spalte(-1)
{
	table = new QTableWidget(this);
	QGridLayout *l = new QGridLayout(this);
	l->addWidget(table,0,0);
	aDep = new QAction(tr("mit Spalte verknpfen"),this);
	connect(aDep,SIGNAL(triggered()),this,SLOT(depAction()));
	aUnDep = new QAction(tr("Verknpfung aufheben"),this);
	connect(aUnDep,SIGNAL(triggered()),this,SLOT(undepAction()));
	setContextMenuPolicy(Qt::DefaultContextMenu);
}


StatistikTabelle::~StatistikTabelle()
{}


void StatistikTabelle::setSpalten(const QVector<int> &s)
{
	Spalten=s;
	updateDiagram();
}

void StatistikTabelle::setModel(QStandardItemModel *m)
{
	if (_model) disconnect(_model,SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	_model = m;
	connect(_model,SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	updateDiagram();
}

void StatistikTabelle::updateDiagram()
{
	if (!_model || Spalten.size() ==0) return;


	table->setRowCount(7);    // Anzahl der Statistikfunktionen
	table->setColumnCount(Spalten.size());    // Anzahl der Spalten
	for (uint j=0;j<Spalten.size();++j)
	{ // Spaltenueberschriften uebernehmen
		QString h = _model->headerData(Spalten[j],Qt::Horizontal).toString();
		table->setHorizontalHeaderItem(j,new QTableWidgetItem(h));
		//table->setColumnWidth ( j,50 );
	}

	// Zeilenbeschriftung
	table->setVerticalHeaderItem(0,new QTableWidgetItem("Min"));
	table->setVerticalHeaderItem(1,new QTableWidgetItem("Max"));
	table->setVerticalHeaderItem(2,new QTableWidgetItem("Median"));
	table->setVerticalHeaderItem(3,new QTableWidgetItem("Summe"));
	table->setVerticalHeaderItem(4,new QTableWidgetItem("Durchschnitt"));
	table->setVerticalHeaderItem(5,new QTableWidgetItem("Leer %"));
	table->setVerticalHeaderItem(6,new QTableWidgetItem("Werte %"));

	// Spaltenstatistiken berechnen
	for (uint i=0;i<Spalten.size();++i)
	{
		QVector<double> sp;
		for (int j=0;j<_model->rowCount();++j)
		{
			if( !input_filter.isNull() && input_filter == _model->data(_model->index(j,i))) continue;
			if( filter_spalte>=0 && filter_wert!=_model->data(_model->index(j,filter_spalte)).toString()) continue;
			sp << _model->data(_model->index(j,Spalten[i])).toDouble();
		}
		std::sort(sp.begin(),sp.end());
		const double min = *std::min_element(sp.begin(),sp.end());
		const double max = *std::max_element(sp.begin(),sp.end());
		const double median = * (sp.begin() +sp.size() /2);
		const double sum = std::accumulate(sp.begin(),sp.end(),0.0);
		const double avg = sum/sp.size();
		const double em = _model->rowCount()-sp.size();
		const double emp = em/_model->rowCount() *100.0;
		const double vp = sp.count() *100.0/_model->rowCount();

		table->setItem(0, i, new QTableWidgetItem(QString::number(min)));
		table->setItem(1, i, new QTableWidgetItem(QString::number(max)));
		table->setItem(2, i, new QTableWidgetItem(QString::number(median)));
		table->setItem(3, i , new QTableWidgetItem(QString::number(sum)));
		table->setItem(4, i , new QTableWidgetItem(QString::number(avg)));
		table->setItem(5, i , new QTableWidgetItem(QString::number(emp)));
		table->setItem(6, i , new QTableWidgetItem(QString::number(vp)));
	}
}

QString StatistikTabelle::toText()
{
	QString out;
	for(int j=0;j<table->columnCount();++j)
	{
		QTableWidgetItem *it = table->horizontalHeaderItem(j);
		out += "\t";
		out += it->text();
	}
	out += "\n";
	for(int i=0;i<table->rowCount();++i)
	{
		out += table->verticalHeaderItem(i)->text();
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			out += "\t";
			out += it->text();
		}
		out += "\n";
	}
	return out;
}

QString StatistikTabelle::toHTML()
{
	QString out="<TABLE width=90%>";
	out += "<TR><TD></TD>";
	for(int j=0;j<table->columnCount();++j)
	{
		QTableWidgetItem *it = table->horizontalHeaderItem(j);
		out += "<TD>"+it->text()+"</TD>";
	}
	out += "</TR>";
	for(int i=0;i<table->rowCount();++i)
	{
		out += "<TR><TD>";
		out += table->verticalHeaderItem(i)->text();
		out += "</TD>";
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			out += "<TD>"+it->text()+"</TD>";
		}
		out += "</TR>";
	}
	out += "</TABLE>";
	return out;
}

// Drag and Drop
 void StatistikTabelle::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
         dragStartPosition = event->pos();
 }


 void StatistikTabelle::mouseMoveEvent(QMouseEvent *event)
 {
     if (!(event->buttons() & Qt::LeftButton))
         return;
     if ((event->pos() - dragStartPosition).manhattanLength()
          < QApplication::startDragDistance())
         return;

     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;

	TextInhalt *ti = dynamic_cast<TextInhalt*>(this);
	if(ti)
	{
		// Get Text
		mimeData->setHtml(ti->toHTML());
	}
	else
	{
     // Get Image
     QPixmap px = QPixmap::grabWidget(this);
     mimeData->setImageData(px.toImage());
	}
     //mimeData->setData(mimeType, data);
     drag->setMimeData(mimeData);

     Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
 }

void StatistikTabelle::setDepFilter(QString Filter_Wert,int Filter_Spalte)
{
	filter_wert = Filter_Wert;
	filter_spalte = Filter_Spalte;
	updateDiagram();
}


/*!
    \fn StatistikTabelle::setOutputFilter(const QString& fv)
 */
void StatistikTabelle::setOutputFilter(const QVariant& fv)
{
	output_filter = fv;
	updateDiagram();
}


/*!
    \fn StatistikTabelle::setInputFilter(const QString& fv)
 */
void StatistikTabelle::setInputFilter(const QVariant& fv)
{
	input_filter = fv;
	updateDiagram();
}


/*!
    \fn StatistikTabelle::depAction()
 */
void StatistikTabelle::depAction()
{
	bool ok;
	int col=-1;
	if(_model) col = ColumnSelectDialog::getColumn(_model);
	if(col<0) return;
	QSet<QString> values;
	for(int i=0;i<_model->rowCount();++i) values << _model->data(_model->index(i,col)).toString();
	
	QString v = QInputDialog::getItem(this,tr("Filter"),tr("Wert nach dem gefiltert werden soll"),values.toList(),0,true,&ok);
	
	if(!ok) return;
	setDepFilter(v,col);
	updateDiagram();
}


/*!
    \fn StatistikTabelle::undepAction()
 */
void StatistikTabelle::undepAction()
{
	setDepFilter(QString(),-1);
}

void StatistikTabelle::contextMenuEvent ( QContextMenuEvent * event )
{
	QMenu m;
	m.addAction(aDep);
	m.addAction(aUnDep);
	m.exec(event->globalPos());
}

void StatistikTabelle::saveData(QDataStream& s,int ver)
{
	if(ver<3) return;
	s << input_filter;
	s << filter_wert << (qint32) filter_spalte;
}

void StatistikTabelle::loadData(QDataStream& s,int ver)
{
	if(ver<3) return;
	s >> input_filter;
	QString v;
	int sp;
	s >> v >> sp;
	setDepFilter(v,sp);
}
