//
// C++ Implementation: tabellendiagramm
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "tabellendiagramm.h"

#include <QTableWidget>
#include <QLabel>
#include <QLayout>
#include <QtDebug>

TabellenDiagramm::TabellenDiagramm(QWidget* parent): Diagram(parent)
{
	QGridLayout *l = new QGridLayout(this);
	lTitle = new QLabel(_title,this);
	lTitle->setAlignment(Qt::AlignCenter);
	tab = new QTableWidget(this);
	l->addWidget(lTitle);
	l->addWidget(tab);
}


TabellenDiagramm::~TabellenDiagramm()
{
}


void TabellenDiagramm::setData(const QVector< double >& d)
{
	data = d; // Daten aktualiesieren
	updateData();
}

void TabellenDiagramm::updateData()
{
	lTitle->setText(_title); // Titel setzen
	int rows,cols; // Zeilen und Spaltenzahl
	if(row_width>0) rows = data.size()/row_width; else rows = 1;
	if(row_width>0) cols = row_width; else cols = data.size();
	
	tab->setRowCount(rows);
	tab->setColumnCount(cols);
	
	for(int i=0;i<labels.count();++i) tab->setHorizontalHeaderItem(i,new QTableWidgetItem(labels[i]));
	for(int i=0;i<colLabels.count();++i) tab->setVerticalHeaderItem(i,new QTableWidgetItem(colLabels[i]));
	
	int row=0,col=0;
	for(int i=0;i<data.size();++i)
	{
		QTableWidgetItem *ni = new QTableWidgetItem(QString::number(data[i])+value_postfix);
		ni->setTextAlignment( Qt::AlignRight );
		tab->setItem(row,col,ni);
		col++;
		if(col>=cols)
		{
			row++;
			col=0;
		}
	}
}

void TabellenDiagramm::paintEvent( QPaintEvent *ev)
{
}

QString TabellenDiagramm::toText()
{
	QString out;
	for(int j=0;j<tab->columnCount();++j)
	{
		QTableWidgetItem *it = tab->horizontalHeaderItem(j);
		out += "\t";
		out += it->text();
	}
	out += "\n";
	for(int i=0;i<tab->rowCount();++i)
	{
		QTableWidgetItem *hit = tab->verticalHeaderItem(i);
		if(hit) out += hit->text();
		for(int j=0;j<tab->columnCount();++j)
		{
			QTableWidgetItem *it = tab->item(i,j);
			out += "\t";
			out += it->text();
		}
		out += "\n";
	}
	return out;
}

QString TabellenDiagramm::toHTML()
{
	QString out="<TABLE>";
	out += "<TR><TD></TD>";
	for(int j=0;j<tab->columnCount();++j)
	{
		QTableWidgetItem *it = tab->horizontalHeaderItem(j);
		out += "<TD>"+it->text()+"</TD>";
	}
	out += "</TR>";
	for(int i=0;i<tab->rowCount();++i)
	{
		out += "<TR><TD>";
		QTableWidgetItem *hit=tab->verticalHeaderItem(i);
		if(hit) out += hit->text();
		out += "</TD>";
		for(int j=0;j<tab->columnCount();++j)
		{
			QTableWidgetItem *it = tab->item(i,j);
			out += "<TD>"+it->text()+"</TD>";
		}
		out += "</TR>";
	}
	out += "</TABLE>";
	return out;
}
